//---------------------------------------------------------------------------
#include <process.h>
#include <errno.h>
#include <iostream>
#include <algorithm> // fill, copy
#include <assert.h>
#include <string>
#include <cstring>
#include <iterator>

using namespace std;

#pragma hdrstop
//---------------------------------------------------------------------------
#pragma argsused

// CONSTANTS
char* JAVA_EXEC = "javaw";

/** Surround every parameter by "quotes". */
struct addQuotes {
  void operator()( char* &w) {
        /* 1st working way
        string temp( w );
        temp = "\"" + temp + "\"";
        w = strcpy( new char[temp.length()], temp.c_str() );
        //*/
        //* 2nd working way
        string* strPtr = new string( w );
        *strPtr = "\"" + *strPtr + "\"";
        w = (char*)strPtr->c_str();
        //*/
        }
};

/** Run JEdit.
 * @author Jakub Holy 2005
 * Tested with Windows 2000
 * REQUIREMENTS:
 * 1. The java executable is called javaw.exe and is on the %PATH%
 * 2. jedit.jar is in the same directory as this program
 */
int main(int argc, char* argv[])
{
        /* Get a name of the directory we are run from because we expect
         * jedit.jar to be there too.
         */
         string pathToJedit( argv[0] );
         string::size_type dirNameEnd = pathToJedit.find_last_of( "\\" );
         // Remove the name of this program (e.g. runjedit.exe) and append there
         // "jedit.jar" instead
         pathToJedit = pathToJedit.substr( 0, dirNameEnd + 1 ) + "jedit.jar";

        /* Set up arguments for exec: javaw = non-command line java executable
         * -reuseview tells to jEdit to use an existing instance of jEdit if any
         */
	char*  myargs[] = { JAVA_EXEC, "-jar", (char*)pathToJedit.c_str() , "-reuseview" };
	const int ADD_PARAMS   = 4; /*== # of items in myargs[]*/
	const int NR_PARAMS    = ADD_PARAMS + (argc-1);  /* (# of params we want) - (this runner program's name) */
        char** argumenty_array = new char*[ NR_PARAMS + 1 ];

        // Append the "command-line arguments" to be handeled over to jEdit
	copy (myargs+0, myargs+ADD_PARAMS, argumenty_array);

	//!!! argv[argc] is NULL and must be copied too!
	copy ( argv+1, argv+argc+1, argumenty_array + ADD_PARAMS );

        // add quotes to all but the 1st (java executable name)
        for_each( argumenty_array + 1, argumenty_array + NR_PARAMS, addQuotes() );

        //print:
        //copy (argumenty_array, argumenty_array + NR_PARAMS,ostream_iterator<char*> (cout, " "));
	//cout << endl;

        /*
        * Execute the command = run javaw
        */
	assert( sizeof(char*)* NR_PARAMS < 260); // for security ...
        cout << "\nRunjedit is executing:\n > ";
                copy( argumenty_array, argumenty_array + NR_PARAMS,
                      ostream_iterator<char*> (cout, " "));
        cout << endl;
	if (  execvp( argumenty_array[0], argumenty_array )  == -1 )
        {
                //en error happened ...
                  char * es;
                  switch (errno){
                        case EACCES: es = "You haven't rights to run this"; break;
                        case EMFILE: es = "Too many opened files"; break;
                        case ENOENT: es = "Application not found"; break;
                        case ENOEXEC: es = "Exec format error"; break;
                        case ENOMEM: es = "Unsuffisient memory"; break;
                        default: es = "An unspecified error"; break;
                  }

                cout << "\nSorry, I cannot run\n > ";
                copy( argumenty_array, argumenty_array + NR_PARAMS,
                      ostream_iterator<char*> (cout, " "));
                cout << "\nbecause: " << es << endl;
        }
	/* NEVER REACHED if exec doesn't fail ... */
        return 0;
}

//---------------------------------------------------------------------------

